// $Id$

//=============================================================================
/**
 *  @file    metadata.h
 *
 *  @author Fukasawa Mitsuo
 *
 *
 *    Copyright (C) 1998-2003 BEE Co.,Ltd. All rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
//=============================================================================

#ifndef JYUGEM_BEE_METADATA_H
#define JYUGEM_BEE_METADATA_H

#include "jyugem/bee.h"

namespace jyugem { namespace bee
{

//
// Meta Data class
//
class BEE_Export metadata : public nameable
{
    friend class value;
protected:
    int    m_t;       // bee type
    size_t m_bytes;   // quantity

public:
    // Constructor
    metadata(int type, size_t bytes, const std::string& name)
        : nameable(name), m_t(type), m_bytes(bytes) {}

    // Destructor
    virtual ~metadata() {}

    inline int type() const   { return m_t; }
    inline const std::string& name() const { return this->nameable::uniqueName(); }
    inline size_t size() { return m_bytes; }

    virtual bool equal(const value& lhs, const value& rhs) = 0;
    virtual bool less(const value& lhs, const value& rhs) = 0;

    virtual size_t length(const value& v) const = 0;

    virtual bool getBool(const value& v) const = 0;
    virtual int  getInt(const value& v) const = 0;
    virtual UINT getUInt(const value& v) const = 0;
    virtual int  setInt(value& lv, int v) = 0;

    virtual int  set(value& v, const value& val);
    virtual int  set(value& v, const string& valstr);
    virtual int  set(value& v, const BYTE * buf, size_t q,
                     int order = ENDIAN) = 0;
    virtual int  get(const value& v, std::string& retstr) = 0;
    virtual int  get(const value& v, BYTE * buf, size_t q,
                     int order = ENDIAN) = 0;

    virtual int  make(value& outv) = 0;

    static metadata * instance(int type);                  // primitive

//
protected:
    virtual void copy(value& lhs, const value& rhs) = 0;
};

//
//
//
template<typename T>
class metadata_t : public metadata
{
public:
    // Constructor
    metadata_t<T>(int type, size_t bytes, const std::string& name)
        : metadata(type, bytes, name) {}

    virtual ~metadata_t() {}

    virtual bool equal(const value& lhs, const value& rhs);
    virtual bool less(const value& lhs, const value& rhs);

    virtual size_t length(const value& v) const;
    virtual bool getBool(const value& v) const;
    virtual int  getInt(const value& v) const;
    virtual UINT getUInt(const value& v) const;
    virtual int  setInt(value& lv, int v);

    virtual int  set(value& lhs, const value& rhs);
    virtual int  set(value& lhs, const string& vs);
    virtual int  set(value& lhs, const BYTE * buf, size_t q,
                     int order = ENDIAN);
    virtual int  get(const value& v, string& retstr);
    virtual int  get(const value& v, BYTE * buf, size_t q,
                     int order = ENDIAN);

    virtual int  make(value& outv);
//
protected:
    virtual void copy(value& lhs, const value& rhs) = 0;
};

//
//
//
class meta_bool : public metadata_t<bool>
{
public:
    meta_bool() : metadata_t<bool>(BEE_BOOLEAN, sizeof(bool), _TX("bool")) {}

    virtual bool getBool(const value& vp) const;
};

//
//
//
class meta_char : public metadata_t<char>
{
public:
    meta_char() : metadata_t<char>(BEE_CHAR, sizeof(char), _TX("char")) {}
};

//
//
//
class meta_byte : public metadata_t<BYTE>
{
public:
    meta_byte() : metadata_t<BYTE>(BEE_BYTE, sizeof(BYTE), _TX("byte")) {}
};

//
//
//
class meta_short : public metadata_t<short>
{
public:
    meta_short()
        : metadata_t<short>(BEE_SHORT, sizeof(short), _TX("short")) {}
};

//
//
//
class meta_ushort : public metadata_t<USHORT>
{
public:
    meta_ushort()
        : metadata_t<ushort>(BEE_USHORT, sizeof(USHORT), _TX("ushort")) {}
    };

//
//
//
class meta_int : public metadata_t<int>
{
public:
    meta_int() : metadata_t<int>(BEE_INT, sizeof(int), _TX("int")) {}
};

//
//
//
class meta_uint : public metadata_t<UINT>
{
public:
    meta_uint()
        : metadata_t<uint>(BEE_UINT, sizeof(UINT), _TX("uint")) {}
};

//
//
//
class meta_longlong : public metadata_t<LONGLONG>
{
public:
    meta_longlong()
        : metadata_t<LONGLONG>(BEE_LONGLONG, sizeof(LONGLONG),
                               _TX("longlong")) {}
};

//
//
//
class meta_ulonglong : public metadata_t<ULONGLONG>
{
public:
    meta_ulonglong()
        : metadata_t<ULONGLONG>(BEE_ULONGLONG, sizeof(ULONGLONG),
                                _TX("ulonglong")) {}
};

//
//
//
class meta_float : public metadata_t<float>
{
public:
    meta_float() : metadata_t<float>(BEE_FLOAT, sizeof(float), _TX("float")) {}
};

//
//
//
class meta_double : public metadata_t<double>
{
public:
    meta_double()
        : metadata_t<double>(BEE_DOUBLE, sizeof(double), _TX("double")) {}
};

//
//
//
class meta_binary : public metadata_t<binary>
{
public:
    meta_binary()
        : metadata_t<binary>(BEE_BINARY, sizeof(BYTE), _TX("binary")) {}
};

//
//
//
class meta_string : public metadata_t<std::string>
{
public:
    meta_string()
        : metadata_t<std::string>(BEE_STRING, sizeof(BCHAR), _TX("string")) {}
    };

//
//
//
class meta_array : public metadata_t<array_val>
{
public:
    meta_array()
        : metadata_t<array_val>(BEE_ARRAY, 0, _TX("array")) {}
};

//
//
//
class meta_struct : public metadata_t<struct_val>
{
public:
    meta_struct()
        : metadata_t<struct_val>(BEE_STRUCT, 0, _TX("struct")) {}
};

//
//
//
class plural_char : public metadata_t<array_char>
{
public:
    plural_char()
        : metadata_t<array_char>((BEE_PLURAL | BEE_CHAR),
                                 sizeof(char), _TX("char[%d]")) {}
};

//
//
//
class plural_short : public metadata_t<array_short>
{
public:
    plural_short()
        : metadata_t<array_short>((BEE_PLURAL | BEE_SHORT),
                                  sizeof(short), _TX("short[%d]")) {}
};

//
//
//
class plural_ushort : public metadata_t<array_ushort>
{
public:
    plural_ushort()
        : metadata_t<array_ushort>((BEE_PLURAL | BEE_USHORT),
                                   sizeof(ushort), _TX("ushort[%d]")) {}
};

//
//
//
class plural_int : public metadata_t<array_int>
{
public:
    plural_int() : metadata_t<array_int>((BEE_PLURAL | BEE_INT),
                                          sizeof(int), _TX("int[%d]")) {}
};

//
//
//
class plural_uint : public metadata_t<array_uint>
{
public:
    plural_uint() : metadata_t<array_uint>((BEE_PLURAL | BEE_UINT),
                                           sizeof(uint), _TX("uint[%d]")) {}
};

//
//
//
class plural_longlong : public metadata_t<array_longlong>
{
public:
    plural_longlong()
        : metadata_t<array_longlong>((BEE_PLURAL | BEE_LONGLONG),
                                     sizeof(LONGLONG),
                                     _TX("longlong[%d]")) {}
};

//
//
//
class plural_ulonglong : public metadata_t<array_ulonglong>
{
public:
    plural_ulonglong()
        : metadata_t<array_ulonglong>((BEE_PLURAL | BEE_ULONGLONG),
                                      sizeof(ULONGLONG),
                                      _TX("ulonglong[%d]")) {}
};

//
//
//
class plural_float : public metadata_t<array_float>
{
public:
    plural_float()
        : metadata_t<array_float>((BEE_PLURAL | BEE_FLOAT),
                                  sizeof(float), _TX("float[%d]")) {}
};

//
//
//
class plural_double : public metadata_t<array_double>
{
public:
    plural_double()
        : metadata_t<array_double>((BEE_PLURAL | BEE_DOUBLE),
                                   sizeof(double), _TX("double[%d]")) {}
};


} } // *namespace*

#endif  /* METADATA_H */
